
/*
 * pbrt source code Copyright(c) 1998-2005 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For educational use only; commercial use expressly forbidden.
 * NO WARRANTY, express or implied, for this software.
 * (See file License.txt for complete license)
 */

// water.cpp*
#include "pbrt.h"
#include "texture.h"
#include "paramset.h"
// WindyTexture Declarations
template <class T> class WaterTexture : public Texture<T> {
public:
	// WaterTexture Public Methods
	~WaterTexture() {
		delete mapping;
	}
	WaterTexture(TextureMapping3D* map, float fm) {
		mapping = map;
		frequencyModifier = fm;
	}
	T Evaluate(const DifferentialGeometry &dg) const {
		Vector dpdx, dpdy;
		Point P = mapping->Map(dg, &dpdx, &dpdy);
		P = P * frequencyModifier;
		//P = P * 0.05f;
		//P = P * 0.1f;
		float windStrength = FBm(P, .1f * dpdx, .1f * dpdy, .5f, 3);
		float waveHeight = 0.5f * FBm(P, dpdx, dpdy, .5f, 6);
		return fabsf(windStrength) * waveHeight;


		/*
		Vector dpdx, dpdy;
		Point P = mapping->Map(dg, &dpdx, &dpdy);
		float total = 0;

		for(int a=0;a<4;a++)
		{
			float ux,uy,uz;
			ux = 10.0f * P.x;
			uy = 10.0f * P.y;
			uz = 10.0f * P.z;

			float scale = 1 / pow(2.f, float(3-a));
			//noise from [-1,1] to [0,1]
			float noise = (1.f + Noise(ux*scale, uy*scale, uz*scale))/2.f;
			total += noise / pow(2.f, float(a));
		}
		
		float cloud_cover = 0.1f;
		float sharpness = 0.70f;
		float c = 255.0f*total - (255.f - cloud_cover*255.f);
		if(c < 0) c=0;
		float value = 1.f - ((float)(pow(sharpness, c)));

		return value;
		*/
	}
private:
	// WaterTexture Private Data
	TextureMapping3D *mapping;
	float frequencyModifier;
};
// WaterTexture Method Definitions
extern "C" DLLEXPORT Texture<float> * CreateFloatTexture(const Transform &tex2world,
		const TextureParams &tp) {
	// Initialize 3D texture mapping _map_ from _tp_
	TextureMapping3D *map = new IdentityMapping3D(tex2world);
	float fm = tp.FindFloat("frequencymodifier", 0.05f);
	return new WaterTexture<float>(map, fm);
}

extern "C" DLLEXPORT Texture<Spectrum> * CreateSpectrumTexture(const Transform &tex2world,
		const TextureParams &tp) {
	// Initialize 3D texture mapping _map_ from _tp_
	TextureMapping3D *map = new IdentityMapping3D(tex2world);
	float fm = tp.FindFloat("frequencymodifier", 0.05f);
	return new WaterTexture<Spectrum>(map, fm);
}
