
/*
 * pbrt source code Copyright(c) 1998-2005 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For educational use only; commercial use expressly forbidden.
 * NO WARRANTY, express or implied, for this software.
 * (See file License.txt for complete license)
 */

// volumegrid.cpp*
#include "volume.h"

// VolumeGrid Declarations
class VolumeGrid : public DensityRegion {
public:
	// VolumeGrid Public Methods
	VolumeGrid(const Spectrum &sa, const Spectrum &ss, float gg,
	 		const Spectrum &emit, const BBox &e, const Transform &v2w,
			int nx, int ny, int nz, const float *d);
	~VolumeGrid() { delete[] density; }
	BBox WorldBound() const { return WorldToVolume.GetInverse()(extent); }
	bool IntersectP(const Ray &r, float *t0, float *t1) const {
		Ray ray = WorldToVolume(r);
		return extent.IntersectP(ray, t0, t1);
	}
	float Density(const Point &Pobj) const;
	float D(int x, int y, int z) const {
		x = Clamp(x, 0, nx-1);
		y = Clamp(y, 0, ny-1);
		z = Clamp(z, 0, nz-1);
		return density[z*nx*ny + y*nx + x];
	}
private:
	void PopulateDensities();
	// VolumeGrid Private Data
	float *density;
	const int nx, ny, nz;
	const BBox extent;

};

// VolumeGrid Method Definitions
VolumeGrid::VolumeGrid(const Spectrum &sa,
		const Spectrum &ss, float gg,
 		const Spectrum &emit, const BBox &e,
		const Transform &v2w,
		int x, int y, int z, const float *d)
	: DensityRegion(sa, ss, gg, emit, v2w),
	nx(x), ny(y), nz(z), extent(e) {
	density = new float[nx*ny*nz];
	if(d != 0)
	{
		memcpy(density, d, nx*ny*nz*sizeof(float));
	} else {
		PopulateDensities();
	}
}

void VolumeGrid::PopulateDensities()
{
	//float xstep = (extent.pMax.x - extent.pMin.x)/float(nx);
	//float ystep = (extent.pMax.y - extent.pMin.y)/float(ny);
	//float zstep = (extent.pMax.z - extent.pMin.z)/float(nz);

	//float curx = extent.pMin.x;
	//float cury = extent.pMin.y;
	//float curz = extent.pMin.z;

	//float totx = (extent.pMax.x - extent.pMin.x);
	//float toty = (extent.pMax.y - extent.pMin.y);
	//float totz = (extent.pMax.z - extent.pMin.z);

	float xstep = 10.0f/float(nx);
	float ystep = 10.0f/float(ny);
	float zstep = 10.0f/float(nz);

	float curx = -5.0f;
	float cury = -5.0f;
	float curz = -5.0f;

	float totx = 10.0f;
	float toty = 10.0f;
	float totz = 10.0f;

	
	if(totx < 0) totx = -totx;
	if(totx == 0) totx = 1;
	if(toty < 0) toty = -toty;
	if(toty == 0) toty = 1;
	if(totz < 0) totz = -totz;
	if(totz == 0) totz = 1;


	//float persistence = 0.5f;
	//float cloud_sharpness = 0.95f;
	//float cloud_cover = 0.5f;


	for(int z=0;z<nz;z++)
	{
		//cury = extent.pMin.y;
		cury = -5.0f;
		for(int y=0;y<ny;y++)
		{
			//curx = extent.pMin.x;
			curx = -5.0f;
			for(int x=0;x<nx;x++)
			{
				//implement octaves perlin noise here??
				float total = 0;

				for(int a=0;a<4;a++)
				{
					float ux,uy,uz;
					ux = 33.0f * curx/totx;
					uy = 33.0f * cury/toty;
					uz = 33.0f * curz/totz;
					//ux = 55.0f * curx/totx;
					//uy = 55.0f * cury/toty;
					//uz = 55.0f * curz/totz;

					float scale = 1 / pow(2.f, float(3-a));
					//noise from [-1,1] to [0,1]
					float noise = (1.f + Noise(ux*scale, uy*scale, uz*scale))/2.f;
					total += noise / pow(2.f, float(a));
				}
				
				float cloud_cover = 0.001f;
				float sharpness = 0.97f;
				//float c = total - (1.f - cloud_cover);
				float c = 255.0f*total - (255.f - cloud_cover*255.f);
				if(c < 0) c=0;
				//density[z*nx*ny + y*nx + x] = 1.f - ((float)(pow(sharpness, c)));
				density[z*nx*ny + y*nx + x] = 1.f - ((float)(pow(sharpness, c)));


//				density[z*nx*ny + y*nx + x] *= 4.5f;

				if(y > ny/2)
				{
					density[z*nx*ny + y*nx + x] *= float(y)/float(ny) * 0.5f;
				} else {
					density[z*nx*ny + y*nx + x] *= 4.5f;
				}

				/*
				for(int i=0;i<4;i++)
				{
					float frequency = pow(2.0f, i);
					float amplitude = pow(persistence, i);
					float ux,uy,uz;
					ux = frequency * 256.0f * curx/totx;
					uy = frequency * 256.0f * cury/toty;
					uz = frequency * 256.0f * curz/totz;

					float noise = (1.f + Noise(ux, uy, uz)) / 2.f;
					total = total + noise / frequency;	//used to be total + (0.5f + noise) / amplitude
				}
				
				//total = min(total, 1.f);
				printf("(%d, %d, %d) = %f, ", x, y, z, total);
				float c = total - cloud_cover;
				if(c < 0) c = 0;

				density[z*nx*ny + y*nx + x] = 3.0f * (1 - pow(cloud_sharpness, 10.f * c));
				printf("%f\n", density[z*nx*ny + y*nx + x]);
				*/
				curx += xstep;
			}
			cury += ystep;
		}
		curz += zstep;
	}

	printf("Done Generating Noise\n");
}


float VolumeGrid::Density(const Point &Pobj) const {
	if (!extent.Inside(Pobj)) return 0;
	// Compute voxel coordinates and offsets for _Pobj_
	float voxx = (Pobj.x - extent.pMin.x) /
		(extent.pMax.x - extent.pMin.x) * nx - .5f;
	float voxy = (Pobj.y - extent.pMin.y) /
		(extent.pMax.y - extent.pMin.y) * ny - .5f;
	float voxz = (Pobj.z - extent.pMin.z) /
		(extent.pMax.z - extent.pMin.z) * nz - .5f;
	int vx = Floor2Int(voxx);
	int vy = Floor2Int(voxy);
	int vz = Floor2Int(voxz);
	float dx = voxx - vx, dy = voxy - vy, dz = voxz - vz;
	// Trilinearly interpolate density values to compute local density
	float d00 = Lerp(dx, D(vx, vy, vz),     D(vx+1, vy, vz));
	float d10 = Lerp(dx, D(vx, vy+1, vz),  D(vx+1, vy+1, vz));
	float d01 = Lerp(dx, D(vx, vy, vz+1),  D(vx+1, vy, vz+1));
	float d11 = Lerp(dx, D(vx, vy+1, vz+1),D(vx+1, vy+1, vz+1));
	float d0 = Lerp(dy, d00, d10);
	float d1 = Lerp(dy, d01, d11);
	return Lerp(dz, d0, d1);
}
extern "C" DLLEXPORT VolumeRegion *CreateVolumeRegion(const Transform &volume2world,
		const ParamSet &params) {
	// Initialize common volume region parameters
	Spectrum sigma_a = params.FindOneSpectrum("sigma_a", 0.1);
	Spectrum sigma_s = params.FindOneSpectrum("sigma_s", 0.1);
	float g = params.FindOneFloat("g", 0.);
	Spectrum Le = params.FindOneSpectrum("Le", 0.);
	Point p0 = params.FindOnePoint("p0", Point(0,0,0));
	Point p1 = params.FindOnePoint("p1", Point(1,1,1));
	int nitems;
	int nx = params.FindOneInt("nx", 1);
	int ny = params.FindOneInt("ny", 1);
	int nz = params.FindOneInt("nz", 1);

	const float *data = params.FindFloat("density", &nitems);
	if (!data) {
		//Error("No \"density\" values provided for volume grid?");
		//return NULL;
	} else {
		if (nitems != nx*ny*nz) {
			Error("VolumeGrid has %d density values but nx*ny*nz = %d",
				nitems, nx*ny*nz);
			return NULL;
		}
	}
	return new VolumeGrid(sigma_a, sigma_s, g, Le, BBox(p0, p1),
		volume2world, nx, ny, nz, data);
}
